import * as React from 'react';

export const ActionButtonDerivedElementComponent = 'button' as const;

export type ActionButtonDerivedElement = HTMLElementTagNameMap[typeof ActionButtonDerivedElementComponent];

export interface ActionButtonProps extends React.HTMLProps<ActionButtonDerivedElement> {
	primary?: boolean;
}

type ButtonType = 'button' | 'reset' | 'submit' | undefined;

export const ActionButton = React.forwardRef<ActionButtonDerivedElement, ActionButtonProps>(({
	primary,
	children,
	type,
	className,
	...etcProps
}, forwardedRef) => {
	return (
		<button
			{...etcProps}
			ref={forwardedRef}
			type={type as ButtonType}
			className={`h-12 px-4 border rounded ${className ?? ''}`.trim()}
		>
			<span>
				{children}
			</span>
		</button>
	)
});

ActionButton.displayName = 'ActionButton';
