import * as React from 'react';

export const MultilineTextInputDerivedElementComponent = 'textarea' as const;

export type MultilineTextInputDerivedElement = HTMLElementTagNameMap[typeof MultilineTextInputDerivedElementComponent];

export interface MultilineTextInputProps extends Omit<React.HTMLProps<MultilineTextInputDerivedElement>, 'label'> {
	label?: React.ReactNode;
}

export const MultilineTextInput = React.forwardRef<MultilineTextInputDerivedElement, MultilineTextInputProps>(({
	className = 'inline-block',
	style,
	label,
	rows = 3,
	...etcProps
}, forwardedRef) => {
	return (
		<div
			className={className}
			style={style}
		>
			<label className="relative block">
				<span className="after:block pointer-events-none absolute top-0 left-0 text-xs px-1">
					{label}
				</span>
				<MultilineTextInputDerivedElementComponent
					{...etcProps}
					ref={forwardedRef}
					rows={rows}
					style={{
						height: `${(rows * 1.5) + 2}rem`
					}}
					className="border rounded min-h-12 px-4 py-4 block w-full resize-y"
				/>
			</label>
		</div>
	);
});

MultilineTextInput.displayName = 'TextInput';
