import * as React from 'react';

export const TextInputDerivedElementComponent = 'input' as const;

export type TextInputDerivedElement = HTMLElementTagNameMap[typeof TextInputDerivedElementComponent];

export interface TextInputProps extends Omit<React.HTMLProps<TextInputDerivedElement>, 'label'> {
	label?: React.ReactNode;
}

export const TextInput = React.forwardRef<TextInputDerivedElement, TextInputProps>(({
	className = 'inline-block',
	style,
	label,
	...etcProps
}, forwardedRef) => {
	return (
		<div
			className={className}
			style={style}
		>
			<label className="relative block">
				<span className="after:block pointer-events-none absolute top-0 left-0 text-xs px-1">
					{label}
				</span>
				<TextInputDerivedElementComponent
					{...etcProps}
					ref={forwardedRef}
					className="border rounded min-h-12 h-12 px-4 block w-full"
				/>
			</label>
		</div>
	);
});

TextInput.displayName = 'TextInput';
