import * as React from 'react';
import { NextApiResponse } from '../../common/types';

export interface UseResponseParams {
	res: NextApiResponse;
}

export const useResponse = (params: UseResponseParams) => {
	const { res } = params;
	const [response, setResponse] = React.useState<Response | undefined>(
		res.body ? new Response(res.body as unknown as BodyInit) : undefined,
	);
	const [loading, setLoading] = React.useState(false);

	const invalidate = React.useCallback(() => {
		setResponse(undefined);
		setLoading(true);
	}, []);

	const refresh = React.useCallback((newResponse: Response) => {
		setResponse(newResponse);
		setLoading(false);
	}, []);

	return React.useMemo(() => ({
		response,
		refresh,
		invalidate,
		loading,
	}), [
		response,
		refresh,
		invalidate,
		loading,
	]);
};
