import { NextApiHandler, PageConfig } from 'next';
import { NextApiRequest as DefaultNextApiRequest } from 'next/dist/shared/lib/utils';
import { ActionWrapperOptions } from './common';
import { METHODS_WITH_BODY } from '../../common/constants';
import { deserializeFormObjectBody } from '../../utils/serialization';

/**
 * Wraps the API handler's `config` export to support all content types.
 * @param customConfig
 */
export const getApiConfig = (customConfig = {} as PageConfig) => ({
	api: {
		...(customConfig.api ?? {}),
		bodyParser: false,
	},
});

/**
 * Wraps the API handler to support script.
 * @param options
 */
export const wrapApiHandler = (options: ActionWrapperOptions): NextApiHandler => async (
	req,
	res,
) => {
	const reqMut = req as unknown as Record<string, unknown>;
	if (METHODS_WITH_BODY.includes(req.method?.toUpperCase() as typeof METHODS_WITH_BODY[number])) {
		reqMut.body = await deserializeFormObjectBody({
			req,
			deserializers: options.deserializers,
		});
	}
	return options.fn(reqMut as unknown as DefaultNextApiRequest, res);
};
