import { ServerResponse } from 'http';
import { NextApiResponse as DefaultNextApiResponse } from 'next/dist/shared/lib/utils';
import { serialize } from 'seroval';
import { ENCTYPE_APPLICATION_JSON, ENCTYPE_APPLICATION_OCTET_STREAM } from '../common/enctypes';
import {
	ACTION_STATUS_CODE,
	DEFAULT_RESPONSE_STATUS_CODE,
} from './constants';
import { CONTENT_TYPE_HEADER_KEY, LOCATION_HEADER_KEY } from '../common/constants';

// for client-side
class DummyServerResponse {}

const EffectiveServerResponse = ServerResponse ?? DummyServerResponse;

export class IceformNextServerResponse
	extends EffectiveServerResponse
	implements DefaultNextApiResponse {
	data?: unknown;

	contentType?: string;

	location?: string;

	private readonly revalidateResponse = Promise.resolve(undefined);

	constructor(...args: ConstructorParameters<typeof EffectiveServerResponse>) {
		super(...args);
		this.statusCode = DEFAULT_RESPONSE_STATUS_CODE;
	}

	setHeader(name: string, value: number | string | readonly string[]): this {
		super.setHeader(name, value);

		if (name.toLowerCase() === CONTENT_TYPE_HEADER_KEY) {
			this.contentType = value.toString();
		}

		if (name.toLowerCase() === LOCATION_HEADER_KEY) {
			this.location = value.toString();
		}

		return this;
	}

	clearPreviewData(): DefaultNextApiResponse {
		// unused
		return this as unknown as DefaultNextApiResponse;
	}

	private static serialize(body: unknown) {
		return serialize(body);
	}

	json(body: unknown): void {
		this.contentType = ENCTYPE_APPLICATION_JSON;
		this.data = IceformNextServerResponse.serialize(body);
	}

	revalidate(): Promise<void> {
		// unused
		return this.revalidateResponse;
	}

	send(body: unknown): void {
		// xtodo: how to transfer binary response in a more compact way?
		// > we let seroval handle this for now
		if (typeof body === 'undefined' || body === null) {
			return;
		}

		if (body instanceof Buffer) {
			this.contentType = ENCTYPE_APPLICATION_OCTET_STREAM;
		}

		this.data = IceformNextServerResponse.serialize(body);
	}

	setDraftMode(): DefaultNextApiResponse {
		// unused
		return this as unknown as DefaultNextApiResponse;
	}

	setPreviewData(): DefaultNextApiResponse {
		// unused
		return this as unknown as DefaultNextApiResponse;
	}

	status(statusCode: number): DefaultNextApiResponse {
		this.statusCode = statusCode;
		return this as unknown as DefaultNextApiResponse;
	}

	strictContentLength = true;

	redirect(...args: [string] | [number, string]): DefaultNextApiResponse {
		const [arg1, arg2] = args;
		if (typeof arg1 === 'number' && typeof arg2 === 'string') {
			this.statusCode = arg1;
			this.setHeader('Location', arg2);
		} else if (typeof arg1 === 'string') {
			this.statusCode = ACTION_STATUS_CODE;
			this.setHeader('Location', arg1);
		}
		return this as unknown as DefaultNextApiResponse;
	}
}
